__includes ["stateMachines.nls" "communication.nls"]
extensions [table]

breed [obstacles obstacle] 
breed [samples sample ]
breed [collectors collector]
breed [bases base]

bases-own [
  incoming-queue ;; Message Queue
  active-states ;; The current state of the world
  active-states-code ;; The code defined for the specific state.
  active-machines ;;
  active-machine-names ;;
  capacity 
  requests
]

collectors-own [
  incoming-queue ;; Message Queue
  active-states ;; The current state of the world
  active-states-code ;; The code defined for the specific state.
  active-machines ;;
  active-machine-names ;;
  ]

to setup
  ;; initialising the environment
  clear-all
  reset-ticks 
  setup-base  
  setup-obstacles
  setup-samples
  setup-collectors
end


to setup-base
  create-bases 1 [
    set shape "spacecraft"
    set color yellow
    setxy 0 0
    set capacity 0 
    set requests [] 
    initialise-state-machine ;; init state machine 
    init-com-agent ;; init message queue
    ]
  create-turtles 1 [set shape "circle 2" set size 5 set color yellow]
end 


;;;;;;;;;;;;; Setting up the various "agents" in the environment
to setup-obstacles
   create-obstacles no-of-obstacles [   
      set shape "tile stones"
      rand-xy-co
      set color blue
      ] 
end 

to setup-samples
   create-samples  no-of-samples [   
      set shape "circle"
      rand-xy-co
      set color red
      ] 
end 

to setup-collectors
   create-collectors no-of-collectors [   
      set shape "bulldozer top"
      rand-xy-co ;; setxy (random screen-edge-x + random (- screen-edge-x) ) (random screen-edge-y + random (- screen-edge-y))   
      set color green
      initialise-state-machine ;; init state machine 
      init-com-agent ;; init message queue
      ] 
end 

;;;;;;;;;;;;;;
to run-experiment
  ask bases [execute-state-machine]
  ask collectors [execute-state-machine]
  tick
end

to-report state-def-of-bases
  report (list 
    state "accepting"
      # when "capacity > 8" do "nothing" goto "not-accepting"
      # when "not empty? requests" do "allow-collector first requests" goto "accepting"
      # when "message-arrived? and message-is \"inform\" " do "collector-finished" goto "accepting"
      # when "message-arrived? and message-is \"query-if\" " do "store-collector get-message" goto "accepting"
      # otherwise do "nothing" goto "accepting"
    end-state 
    state "not-accepting"
      # when "capacity <= 8" do "nothing" goto "accepting"
      # when "message-arrived? and message-is \"query-if\" " do "store-collector get-message" goto "not-accepting"
      # when "message-arrived? and message-is \"inform\" " do "collector-finished" goto "not-accepting"
      # otherwise do "nothing" goto "not-accepting"
  end-state
    )
end 


to collector-finished
   remove-msg
   set capacity capacity - 1 
end 


to store-collector [msg]
  set requests lput msg requests
end

to allow-collector [msg]
 ;; show "allowing collector"
  send add-content "ok" create-reply "inform" msg
  set capacity capacity + 1
  set requests but-first requests
end



to-report state-def-of-collectors
  report (list
    state "patrol"
    # when "detect-obstacle" do "nothing" activate-machine "get-away-from-obstacle 10"
    # on-success "get-away-from-obstacle 10" do "set color yellow" goto "patrol-dummy"
    # on-failure "get-away-from-obstacle 10" do "set color orange" activate-machine "get-away-from-obstacle 15"
    # when "detect-samples" do "pick-samples" goto "approach-base"
    # otherwise do "move-randomly" goto "patrol"
    end-state 
    state "patrol-dummy"
    # for-n-ticks 10 do "set color color + 10" goto "patrol-dummy"
    # otherwise do "nothing" goto "patrol"
    end-state
    state "approach-base"
    # when "detect-obstacle" do "nothing" activate-machine "get-away-from-obstacle 10"
    # when "near-base" do "ask-base" goto "wait-approval"
    # otherwise do "move-towards-base" goto "approach-base"
    end-state
    state "wait-approval"  
    # when "message-arrived? and message-is \"inform\" " do "remove-msg" goto "move-to-base" 
    # otherwise do "nothing" goto "wait-approval"
    end-state
    state "move-to-base"
    # when "detect-obstacle" do "nothing" activate-machine "get-away-from-obstacle 10"
    # when "at-base" do "drop-samples inform-base" goto "patrol"
    # otherwise do "move-towards-base" goto "move-to-base"
    end-state 
    )
end


to-report state-def-of-contractor 
  report (list 
    state "wait-announcement"
    # when "message-arrived? and message-is \"cfp\" " do "propose" goto "waiting-for-answer"
    # when "message-arrived?" do "remove-msg" goto "wait-announcement"
    # otherwise do "nothing" goto "wait-announcement"
    end-state
    state "waiting-for-answer"
    # when "message-arrived? and message-is \"accept-proposal\" " do "nothing" success
    # when "message-arrived? and message-is \"reject-proposal\" " do "nothing" failure 
    # when "message-arrived?" do "remove-msg" goto "waiting-for-answer"
    # otherwise do "nothing" goto "waiting-for-answer"
    end-state
    )
end 

to ask-base 
  send add-receiver [who] of one-of bases add-content "clear for dropping?" create-message "query-if"
end

to inform-base 
  send add-receiver [who] of one-of bases add-content "have dropped load" create-message "inform"
end

to propose 
  send add-content random 100 create-reply "propose" get-message
end

to-report message-is [perf]
  report get-performative get-message-no-remove = perf 
end


to-report state-def-of-get-away-from-obstacle [n]
   report (list 
     state "finding-a-clear-space"
      # when "detect-obstacle" do "change-heading" goto "finding-a-clear-space"
      # for-n-ticks n do "fd 0.2" goto "finding-a-clear-space"
      # otherwise do "nothing" goto "check-space-cleared"
    end-state
    state "check-space-cleared"
       # when "obstacle-around" do "nothing" failure
       # otherwise do "nothing" success
    end-state
    )
end


;; old code with limited functionality.
to execute-behaviour
  if detect-obstacle [change-heading stop]
  if carrying-samples and at-base [drop-samples stop]
  if carrying-samples and not at-base [move-towards-base stop]
  if not carrying-samples and detect-samples and not at-base [pick-samples stop] 
  if not carrying-samples [move-randomly stop]
end  





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; sensors
to-report detect-obstacle
  report (any? other collectors in-cone 1 120 or any? obstacles in-cone 1 120) 
end   

to-report obstacle-around
  report (any? other collectors in-radius 1 or any? obstacles in-radius 1) 
end


to-report at-base
  report any? bases-here
end

to-report detect-samples
 report (any? samples-here) 
end

to-report carrying-samples
  report color = red  
end

to-report near-base
  report distance one-of bases < 10 
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Agent Actions 
to change-heading
  set heading heading + random 90
end

to pick-samples
    set color red
    ask samples-here [die]
end

to move-towards-base
  fd 0.2
  face one-of bases
end

;; droping an item 
to drop-samples    
   set color green
end

;; moving randomly 
to move-randomly
  fd 0.2
  set heading heading + random 45 - random 45
end




;;; Other utilities 
to rand-xy-co
  let x 0
  let y 0
  
  loop [ 
    set x (random max-pxcor + random min-pxcor) 
    set y (random max-pycor + random min-pycor)
    if not any? turtles-on patch-at x y and not ( (abs x) < 3  and (abs y) < 3) [setxy x y stop]
  ]
end 
@#$#@#$#@
GRAPHICS-WINDOW
279
10
909
661
15
15
20.0
1
15
1
1
1
0
1
1
1
-15
15
-15
15
1
1
1
ticks
30.0

BUTTON
118
10
182
43
Setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
10
119
182
152
no-of-collectors
no-of-collectors
0
100
30
1
1
NIL
HORIZONTAL

SLIDER
10
161
182
194
no-of-samples
no-of-samples
0
100
34
1
1
NIL
HORIZONTAL

BUTTON
28
52
91
85
run
run-experiment
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
105
51
183
84
run cont
run-experiment
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
10
203
182
236
no-of-obstacles
no-of-obstacles
0
50
15
1
1
NIL
HORIZONTAL

SWITCH
11
288
185
321
show_messages
show_messages
1
1
-1000

@#$#@#$#@
## WHAT IS IT? 

A NetLogo Simulation of the Steel's Mars Explorer experiment. Uses the TSTATES DSL.

## HOW IT WORKS

This section could explain what rules the agents use to create the overall behavior of the model.

## HOW TO USE IT

This section could explain how to use the model, including a description of each of the items in the interface tab.

## THINGS TO NOTICE

This section could give some ideas of things for the user to notice while running the model.

## THINGS TO TRY

This section could give some ideas of things for the user to try to do (move sliders, switches, etc.) with the model.

## EXTENDING THE MODEL

This section could give some ideas of things to add or change in the procedures tab to make the model more complicated, detailed, accurate, etc.

## NETLOGO FEATURES

This section could point out any especially interesting or unusual features of NetLogo that the model makes use of, particularly in the Procedures tab.  It might also point out places where workarounds were needed because of missing features.

## RELATED MODELS

This section could give the names of models in the NetLogo Models Library or elsewhere which are of related interest.

## CREDITS AND REFERENCES

This section could contain a reference to the model's URL on the web if it has one, as well as any other necessary credits or references.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7500403 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7500403 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7500403 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186
Polygon -7500403 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7500403 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7500403 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7500403 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7500403 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7500403 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -1184463 true false 152 149 77 163 67 195 67 211 74 234 85 252 100 264 116 276 134 286 151 300 167 285 182 278 206 260 220 242 226 218 226 195 222 166
Polygon -16777216 true false 150 149 128 151 114 151 98 145 80 122 80 103 81 83 95 67 117 58 141 54 151 53 177 55 195 66 207 82 211 94 211 116 204 139 189 149 171 152
Polygon -7500403 true true 151 54 119 59 96 60 81 50 78 39 87 25 103 18 115 23 121 13 150 1 180 14 189 23 197 17 210 19 222 30 222 44 212 57 192 58
Polygon -16777216 true false 70 185 74 171 223 172 224 186
Polygon -16777216 true false 67 211 71 226 224 226 225 211 67 211
Polygon -16777216 true false 91 257 106 269 195 269 211 255
Line -1 false 144 100 70 87
Line -1 false 70 87 45 87
Line -1 false 45 86 26 97
Line -1 false 26 96 22 115
Line -1 false 22 115 25 130
Line -1 false 26 131 37 141
Line -1 false 37 141 55 144
Line -1 false 55 143 143 101
Line -1 false 141 100 227 138
Line -1 false 227 138 241 137
Line -1 false 241 137 249 129
Line -1 false 249 129 254 110
Line -1 false 253 108 248 97
Line -1 false 249 95 235 82
Line -1 false 235 82 144 100

bird1
false
0
Polygon -7500403 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7500403 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6459832 true false 150 32 157 162
Polygon -13345367 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7500403 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7500403 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6459832 true false 150 32 157 162
Polygon -13345367 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7500403 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7500403 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6459832 true false 150 32 157 162
Polygon -13345367 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7500403 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7500403 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7500403 true true 45 255 255 255 255 45 45 45

bulldozer top
true
0
Rectangle -7500403 true true 195 60 255 255
Rectangle -16777216 false false 195 60 255 255
Rectangle -7500403 true true 45 60 105 255
Rectangle -16777216 false false 45 60 105 255
Line -16777216 false 45 75 255 75
Line -16777216 false 45 105 255 105
Line -16777216 false 45 60 255 60
Line -16777216 false 45 240 255 240
Line -16777216 false 45 225 255 225
Line -16777216 false 45 195 255 195
Line -16777216 false 45 150 255 150
Polygon -1184463 true true 90 60 75 90 75 240 120 255 180 255 225 240 225 90 210 60
Polygon -16777216 false false 225 90 210 60 211 246 225 240
Polygon -16777216 false false 75 90 90 60 89 246 75 240
Polygon -16777216 false false 89 247 116 254 183 255 211 246 211 211 90 210
Rectangle -16777216 false false 90 60 210 90
Rectangle -1184463 true true 180 30 195 90
Rectangle -16777216 false false 105 30 120 90
Rectangle -1184463 true true 105 45 120 90
Rectangle -16777216 false false 180 45 195 90
Polygon -16777216 true false 195 105 180 120 120 120 105 105
Polygon -16777216 true false 105 199 120 188 180 188 195 199
Polygon -16777216 true false 195 120 180 135 180 180 195 195
Polygon -16777216 true false 105 120 120 135 120 180 105 195
Line -1184463 true 105 165 195 165
Circle -16777216 true false 113 226 14
Polygon -1184463 true true 105 15 60 30 60 45 240 45 240 30 195 15
Polygon -16777216 false false 105 15 60 30 60 45 240 45 240 30 195 15

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7500403 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7500403 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7500403 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7500403 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7500403 true 167 47 159 82
Line -7500403 true 136 47 145 81
Circle -7500403 true true 165 45 8
Circle -7500403 true true 134 45 6
Circle -7500403 true true 133 44 7
Circle -7500403 true true 133 43 8

circle
false
0
Circle -7500403 true true 35 35 230

circle 2
true
0
Circle -7500403 false true 0 0 300

link
true
0
Line -7500403 true 150 0 150 300

link direction
true
0
Line -7500403 true 150 150 30 225
Line -7500403 true 150 150 270 225

person
false
0
Circle -7500403 true true 155 20 63
Rectangle -7500403 true true 158 79 217 164
Polygon -7500403 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7500403 true true 216 83 267 123 248 143 215 107
Polygon -7500403 true true 167 163 145 234 183 234 183 163
Polygon -7500403 true true 195 163 195 233 227 233 206 159

sheep
false
15
Rectangle -1 true true 90 75 270 225
Circle -1 true true 15 75 150
Rectangle -16777216 true false 81 225 134 286
Rectangle -16777216 true false 180 225 238 285
Circle -16777216 true false 1 88 92

spacecraft
true
0
Polygon -7500403 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7500403 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

tile stones
false
0
Polygon -7500403 true true 0 240 45 195 75 180 90 165 90 135 45 120 0 135
Polygon -7500403 true true 300 240 285 210 270 180 270 150 300 135 300 225
Polygon -7500403 true true 225 300 240 270 270 255 285 255 300 285 300 300
Polygon -7500403 true true 0 285 30 300 0 300
Polygon -7500403 true true 225 0 210 15 210 30 255 60 285 45 300 30 300 0
Polygon -7500403 true true 0 30 30 0 0 0
Polygon -7500403 true true 15 30 75 0 180 0 195 30 225 60 210 90 135 60 45 60
Polygon -7500403 true true 0 105 30 105 75 120 105 105 90 75 45 75 0 60
Polygon -7500403 true true 300 60 240 75 255 105 285 120 300 105
Polygon -7500403 true true 120 75 120 105 105 135 105 165 165 150 240 150 255 135 240 105 210 105 180 90 150 75
Polygon -7500403 true true 75 300 135 285 195 300
Polygon -7500403 true true 30 285 75 285 120 270 150 270 150 210 90 195 60 210 15 255
Polygon -7500403 true true 180 285 240 255 255 225 255 195 240 165 195 165 150 165 135 195 165 210 165 255

truck-down
false
0
Polygon -7500403 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8630108 true false 195 75 195 120 240 120 240 75
Polygon -8630108 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7500403 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8630108 true false 90 210 105 225 120 210
Polygon -8630108 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7500403 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8630108 true false 210 210 195 225 180 210
Polygon -8630108 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7500403 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf
false
0
Rectangle -7500403 true true 15 105 105 165
Rectangle -7500403 true true 45 90 105 105
Polygon -7500403 true true 60 90 83 44 104 90
Polygon -16777216 true false 67 90 82 59 97 89
Rectangle -1 true false 48 93 59 105
Rectangle -16777216 true false 51 96 55 101
Rectangle -16777216 true false 0 121 15 135
Rectangle -16777216 true false 15 136 60 151
Polygon -1 true false 15 136 23 149 31 136
Polygon -1 true false 30 151 37 136 43 151
Rectangle -7500403 true true 105 120 263 195
Rectangle -7500403 true true 108 195 259 201
Rectangle -7500403 true true 114 201 252 210
Rectangle -7500403 true true 120 210 243 214
Rectangle -7500403 true true 115 114 255 120
Rectangle -7500403 true true 128 108 248 114
Rectangle -7500403 true true 150 105 225 108
Rectangle -7500403 true true 132 214 155 270
Rectangle -7500403 true true 110 260 132 270
Rectangle -7500403 true true 210 214 232 270
Rectangle -7500403 true true 189 260 210 270
Line -7500403 true 263 127 281 155
Line -7500403 true 281 155 281 192

wolf-left
false
3
Polygon -6459832 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6459832 true true 87 80 79 55 76 79
Polygon -6459832 true true 81 75 70 58 73 82
Polygon -6459832 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6459832 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6459832 true true 116 140 114 189 105 137
Rectangle -6459832 true true 109 150 114 192
Rectangle -6459832 true true 111 143 116 191
Polygon -6459832 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6459832 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6459832 true true 214 134 203 168 192 148
Polygon -6459832 true true 204 151 203 176 193 148
Polygon -6459832 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6459832 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6459832 true true 201 99 214 69 215 99
Polygon -6459832 true true 207 98 223 71 220 101
Polygon -6459832 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6459832 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6459832 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6459832 true true 48 143 58 141
Polygon -6459832 true true 46 136 68 137
Polygon -6459832 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6459832 true true 38 138 66 149
Polygon -6459832 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6459832 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 5.0.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 1.0 0.0
0.0 1 1.0 0.0
0.2 0 1.0 0.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
